@@ -1,5 +1,5 @@
 ''' wrapper functions for common AFNI tasks '''
-import re,subprocess,os,glob
+import re,subprocess,os,glob,copy,tempfile,shutil
 import multiprocessing
 import neural
 import neural as nl
@@ -186,6 +186,13 @@ def cluster(dset,min_distance,min_cluster_size,prefix):
     ''' runs 3dmerge to cluster given dataset '''
     neural.run(['3dmerge','-1clust',min_distance,min_cluster_size,'-prefix',prefix,dset])
 
+def blur(dset,fwhm,prefix=None):
+    ''' runs 3dmerge to blur dataset to given ``fwhm``
+    default ``prefix`` is to suffix ``dset`` with ``_blur%dmm``'''
+    if prefix==None:
+        prefix = suffix(dset,'_blur%dmm'%fwhm)
+    neural.run(['3dmerge','-1blur_fwhm',fwhm,'-prefix',prefix,dset],products=prefix)
+
 def voxel_count(dset,subbrick=0,p=None,positive_only=False,mask=None,ROI=None):
     ''' returns the number of non-zero voxels
     
@@ -238,6 +245,70 @@ def voxel_count(dset,subbrick=0,p=None,positive_only=False,mask=None,ROI=None):
         return count_dict
     return count
 
+def ROIstats(mask,dsets):
+    '''runs 3dROIstats on ``dsets`` using ``mask`` as the mask
+    returns a dictionary with the structure::
+    
+        {
+            ROI: {
+                dset: [
+                    # list of dset subbricks:
+                    {keys: stat},
+                    ...
+                ],
+                ...
+            },
+            ...
+        }
+    
+    keys::
+    
+        :mean:
+        :median:
+        :mode:
+        :nzmean:
+        :nzmedian:
+        :nzmode:
+        :min:
+        :max:
+        :nzmin:
+        :nzmax:
+        :sigma:
+        :nzsigma:
+        :sum:
+        :nzsum:
+    '''
+    out_dict = {}
+    values = [{'Med': 'median', 'Min': 'min', 'Max': 'max', 'NZVoxels': 'nzvoxels', 
+               'NZMean': 'nzmean', 'NZSum': 'nzsum', 'NZSigma': 'nzsigma', 
+               'Mean': 'mean', 'Sigma': 'sigma', 'Mod': 'mode','NZcount':'nzcount'},
+              {'NZMod': 'nzmode', 'NZMed': 'nzmedian', 'NZMax': 'nzmax', 'NZMin': 'nzmin','Mean':'mean'}]
+    options = [['-nzmean','-nzsum','-nzvoxels','-minmax','-sigma','-nzsigma','-median','-mode'],
+               ['-nzminmax','-nzmedian','-nzmode']]
+    if isinstance(dsets,basestring):
+        dsets = [dsets]
+    for i in xrange(2):
+        cmd = ['3dROIstats','-1Dformat','-nobriklab','-mask',mask] + options[i] + dsets
+        out = subprocess.check_output(cmd).split('\n')
+        header = [(values[i][x.split('_')[0]],int(x.split('_')[1])) for x in out[1].split()[1:]]
+        for j in xrange(len(out)/2-1):
+            dset_subbrick = out[(j+1)*2][1:].split()
+            stats = [float(x) for x in out[(j+1)*2+1][1:].split()]
+            for s in xrange(len(stats)):
+                roi = header[s][1]
+                stat_name = header[s][0]
+                stat = stats[s]
+                if roi not in out_dict:
+                    out_dict[roi] = {}
+                if dset_subbrick[0] not in out_dict[roi]:
+                    out_dict[roi][dset_subbrick[0]] = []
+                subbrick = int(dset_subbrick[1])
+                if subbrick>=len(out_dict[roi][dset_subbrick[0]]):
+                    for diff in xrange(subbrick-len(out_dict[roi][dset_subbrick[0]])+1):
+                        out_dict[roi][dset_subbrick[0]].append({})
+                out_dict[roi][dset_subbrick[0]][subbrick][stat_name] = stat
+    return out_dict
+
 _afni_suffix_regex = r"((\+(orig|tlrc|acpc))?\.?(nii|HEAD|BRIK)?(.gz|.bz2)?)(\[\d+\])?$"
 
 def prefix(filename):
@@ -344,7 +415,10 @@ class Decon:
         :stim_files:        dictionary where keys are used as stimulus labels
                             and the values are taken as 1D files
         :stim_times:        same as stim_files, but used as a stim_times file
-        :stim_times_model:  default model to use for each stim_times
+        :models:            dictionary of model names to use for each of the
+                            listed stimuli (optional)
+        :model_default:     default model to use for each ``stim_times`` stimulus if nothing
+                            is listed in ``models``
         :stim_base:         list of names of stimuli (defined either in stim_files or 
                             stim_times) that should be considered in the baseline instead
                             of full model
@@ -355,6 +429,7 @@ class Decon:
         :glts:              dictionary where keys are GLT labels, and the value
                             is a symbolic statement
         :mask:              either a mask file, or "auto", which will use "-automask"
+        :errts:             name of file to save residual time series to
         
         Options that are obvious:
             nfirst (default: 3), censor_file, polort (default: 'A'), tout, vout, rout, prefix
@@ -391,7 +466,8 @@ class Decon:
         self.input_dsets=[]
         self.stim_files={}
         self.stim_times={}
-        self.stim_times_model = 'GAM'
+        self.model_default = 'GAM'
+        self.models = {}
         self.stim_base = []
         self.stim_am1 = []
         self.stim_am2 = []
@@ -408,6 +484,7 @@ class Decon:
         self.reps = None
         self.TR = None
         self.stim_sds = None
+        self.errts = None
     
     def command_list(self):
         '''returns the 3dDeconvolve command as a list
@@ -452,7 +529,12 @@ class Decon:
                 opt = '-stim_times_AM1'
             if stim in self.stim_am2:
                 opt = '-stim_times_AM2'
-            cmd += [opt,stim_num,self.stim_times[stim],self.stim_times_model,'-stim_label',stim_num,stim]
+            cmd += [opt,stim_num,self.stim_times[stim]]
+            if stim in self.models:
+                cmd += [self.models[stim]]
+            else:
+                cmd += [self.model_default]
+            cmd += ['-stim_label',stim_num,stim]
             if stim in self.stim_base:
                 cmd += ['-stim_base',stim_num]
             stim_num += 1
@@ -471,6 +553,9 @@ class Decon:
         if self.rout:
             cmd += ['-rout']
         
+        if self.errts:
+            cmd += ['-errts', self.errts]
+        
         if self.prefix:
             cmd += ['-bucket', self.prefix]
         
@@ -522,7 +607,7 @@ def affine_align(dset_from,dset_to,skull_strip=True,mask=None,affine_suffix='_af
     
     for dset in [dset_from,dset_to]:
         if skull_strip==True or skull_strip==dset:
-            neural.fsl.skull_strip(dset,'_ns')
+            neural.default.skull_strip(dset,'_ns')
         
     mask_use = mask
     if mask:
@@ -652,11 +737,12 @@ def qwarp_apply(dset_from,dset_warp,affine=None,warp_suffix='_warp',master='WARP
         # If the datasets are different orientations, the transform won't be applied correctly
         nl.run(['3dresample','-orient',dset_warp_info.orient,'-prefix',suffix(dset_from,'_reorient'),'-inset',dset_from])
         dset_from = suffix(dset_from,'_reorient')
+    warp_opt = str(dset_warp)
+    if affine:
+        warp_opt += ' ' + affine
     cmd = [
         '3dNwarpApply',
-        '-nwarp', dset_warp]
-    if affine:
-        cmd += ['-affter', affine]
+        '-nwarp', warp_opt]
     cmd += [
         '-source', dset_from,
         '-master',master,
@@ -691,10 +777,11 @@ def skull_strip(dset,out_suffix='_ns'):
         '-ld', '40'
     ],products=suffix(dset,out_suffix))
 
-def align_epi_anat(anatomy,epi_dsets):
+def align_epi_anat(anatomy,epi_dsets,skull_strip_anat=True):
     ''' aligns epis to anatomy using ``align_epi_anat.py`` script
     
     :epi_dsets:       can be either a string or list of strings of the epi child datasets
+    :skull_strip_anat:     if ``True``, ``anatomy`` will be skull-stripped using the default method (in ``neural.default``)
     
     The default output suffix is "_al"
     '''
@@ -706,6 +793,11 @@ def align_epi_anat(anatomy,epi_dsets):
         return
     
     anatomy_use = anatomy
+    
+    if skull_strip_anat:
+        nl.default.skull_strip(anatomy,'_ns')
+        anatomy_use = suffix(anatomy,'_ns')
+        
     if is_nifti(anatomy):
         anatomy_use = afni_copy(anatomy)
     epi_dsets_use = []
@@ -715,7 +807,7 @@ def align_epi_anat(anatomy,epi_dsets):
         else:
             epi_dsets_use.append(dset)
     
-    cmd = ["align_epi_anat.py", "-epi2anat", "-anat", anatomy_use, "-epi_base", "5", "-epi", epi_dsets_use[0]]
+    cmd = ["align_epi_anat.py", "-epi2anat", "-anat_has_skull", "no", "-epi_strip", "3dAutomask","-anat", anatomy_use, "-epi_base", "5", "-epi", epi_dsets_use[0]]
     if len(epi_dsets_use)>1:
         cmd += ['-child_epi'] + epi_dsets_use[1:]
     out = neural.run(cmd)
@@ -724,7 +816,8 @@ def align_epi_anat(anatomy,epi_dsets):
         if is_nifti(dset):
             if dset!=anatomy:
                 dset_nifti = nifti_copy(prefix(dset)+'_al+orig')
-                nl.run(['gzip',dset_nifti])
+                if dset_nifti:
+                    nl.run(['gzip',dset_nifti])
             if dset==anatomy or os.path.exists(suffix(dset,'_al')):
                 for s in ['_al+orig.HEAD','_al+orig.BRIK*','+orig.HEAD','+orig.BRIK*']:
                     for d in glob.glob(prefix(dset) + s):
@@ -756,7 +849,7 @@ def create_censor_file(input_dset,out_prefix=None,fraction=0.1,clip_to=0.1,max_e
     info = dset_info(input_dset)
     outcount = [float(x) for x in subprocess.check_output(['3dToutcount','-fraction','-automask','-polort',str(polort),str(input_dset)]).split('\n') if len(x.strip())>0]
     binary_outcount = [x<fraction for x in outcount]
-    perc_outliers = sum(binary_outcount)/float(info.reps)
+    perc_outliers = 1 - (sum(binary_outcount)/float(info.reps))
     if max_exclude and perc_outliers > max_exclude:
         raise AFNI_Censor_TooManyOutliers('Found %f outliers in dset %s' % (perc_outliers,input_dset))
     if clip_to:
@@ -768,3 +861,53 @@ def create_censor_file(input_dset,out_prefix=None,fraction=0.1,clip_to=0.1,max_e
         out_prefix = prefix(input_dset) + '.1D'
     with open(out_prefix,'w') as f:
         f.write('\n'.join([str(int(x)) for x in binary_outcount]))
+
+def smooth_decon_to_fwhm(decon,fwhm):
+    '''takes an input :class:`Decon` object and uses ``3dBlurToFWHM`` to make the output as close as possible to ``fwhm``
+    returns the final measured fwhm'''
+    tmpdir = tempfile.mkdtemp()
+    try:
+        cwd = os.getcwd()
+        random_files = [re.sub(r'\[\d+\]$','',x) for x in nl.flatten([x for x in decon.__dict__.values() if isinstance(x,basestring) or isinstance(x,list)]+[x.values() for x in decon.__dict__.values() if isinstance(x,dict)])]
+        files_to_copy = [x for x in random_files if os.path.exists(x) and x[0]!='/']
+        # copy crap
+        for file in files_to_copy:
+            try:
+                shutil.copytree(file,tmpdir)
+            except OSError as e:
+                shutil.copy(file,tmpdir)
+            shutil.copy(file,tmpdir)
+        with nl.run_in(tmpdir):
+            if os.path.exists(decon.prefix):
+                os.remove(decon.prefix)
+            old_errts = decon.errts
+            decon.errts = 'residual.nii.gz'
+            decon.prefix = os.path.basename(decon.prefix)
+            # Run once in place to get the residual dataset
+            decon.run()
+            running_reps = 0
+            blur_input = lambda i: 'input_blur-part%d.nii.gz'%(i+1)
+            for i in xrange(len(decon.input_dsets)):
+                dset = decon.input_dsets[i]
+                info = dset_info(dset)
+                residual_dset = 'residual-part%d.nii.gz'%(i+1)
+                nl.run(['3dbucket','-prefix',residual_dset,'%s[%d..%d]'%(decon.errts,running_reps,running_reps+info.reps-1)],products=residual_dset)
+                nl.run(['3dBlurToFWHM','-quiet','-input',dset,'-blurmaster',residual_dset,'-prefix',blur_input(i),'-automask','-FWHM',fwhm],products=blur_input(i))
+                running_reps += info.reps
+            decon.input_dsets = [blur_input(i) for i in xrange(len(decon.input_dsets))]
+            for d in [decon.prefix,decon.errts]:
+                if os.path.exists(d):
+                    try:
+                        os.remove(d)
+                    except:
+                        pass
+            decon.errts = old_errts
+            decon.run()
+            if os.path.exists(decon.prefix):
+                shutil.copy(decon.prefix,cwd)
+            else:
+                nl.notify('Warning: deconvolve did not produce expected file %s' % decon.prefix,level=nl.level.warning)
+    except Exception as e:
+        raise
+    finally:
+        shutil.rmtree(tmpdir,True)
\ No newline at end of file
